/******************************************************************************
 * Login_Command - Handle logging in.
 * $Id: Login_Command.java,v 1.7 2011/06/24 01:42:31 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Login_Command.java,v $
 * Revision 1.7  2011/06/24 01:42:31  dick
 * Wrap some exceptions with nops.
 *
 * Revision 1.6  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.5  2011/06/09 18:11:50  dick
 * Put the unused execute() function at the top, like the other JibsCommands.
 *
 * Revision 1.4  2011/06/06 02:53:16  dick
 * Stop passing the not-so-useful-here Server all around.
 *
 * Revision 1.3  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 */
/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.Socket;
import java.sql.Connection;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.StringTokenizer;
import java.util.TimeZone;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsWriter;

/** The Login_Command will be invoked by jIBS when a user tries to login.
 * Though it adheres to the JibsCommand interface, normal processing is done 
 * by jIBS in a different way.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.7 $ <br> $Date: 2011/06/24 01:42:31 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Login_Command.java">cvs Login_Command.java</a>
 */
public class Login_Command implements JibsCommand {

	/** The execute method is not used by Login_Command */
	public boolean execute(JibsServer unused, Player unused1,
			String unused2, String[] unused3) {
		return true;
	}

	/** Login a player to jIBS
	 * @param jibsServer The jibsServer
	 * @param client
	 * @param out
	 * @param jdbc
	 * @param in
	 * @return
	 */
	public Player login(JibsServer jibsServer, Socket client,
			JibsWriter out, Connection jdbc,
			BufferedReader in) {
		Player player = null;
		String strUser = null;

		try {
			strUser = in.readLine();
		} catch (IOException e) {
			jibsServer.logException(e);
		}

		if (strUser != null) {
			StringTokenizer stoken = new StringTokenizer(strUser);
			String[] MyArgs = new String[5];
			int i = 0;

			while (stoken.hasMoreTokens()) {
				String String_Token = stoken.nextToken();

				if (String_Token.equals("login")) {
					MyArgs[0] = "login";
					i = 1;
				} else if (i == 0) {
					MyArgs[0] = String_Token;
					i = 1;
				} else if (i >= 1) {
					MyArgs[i++] = String_Token;
				}
			}

			if ((i == 1) && MyArgs[0].equalsIgnoreCase("guest")) {
				NewUser_Command cmd = new NewUser_Command(jibsServer, in, out);

				player = cmd.createNewUser(jibsServer, client, "", null);

				if (player != null) {
					player.canCLIP(false);
				}

				return player;
			}

			if (i == 5) {
				// bypass password confirmation
				try {
					player = new Player(jdbc, out,
							MyArgs[1], Integer.parseInt(MyArgs[2]),
							MyArgs[3], MyArgs[4]);
				} catch (NumberFormatException e) {
					jibsServer.logException(e);

					return null;
				}

				player.canCLIP(true);
			} else {
				// normal password confirmation
				out.print("Password: ");
				out.flush();

				String strPassword = null;

				try {
					strPassword = in.readLine();
				} catch (IOException e1) {
					jibsServer.logException(e1);
				}

				player = new Player(jdbc, out, "-", -1,
						strUser, strPassword);
				player.canCLIP(false);
			}
		}

		return player;
	}
	
	public void showLogo(JibsWriter out, int nrTries) {
		BufferedReader inp = null;

		try {
			if (nrTries == 1) {
				String          sName   = JibsServer.getInstance().getResource( "login" );
				File            file    = new File( sName );
				//				FileInputStream fread   = new FileInputStream( file );
				if (file.canRead()) {
					try {
						FileReader      fReader = new FileReader( file );
						inp = new BufferedReader( fReader );
					} catch (Exception e) {
						// e.printStackTrace();
					}
				} else {
					try {
						InputStream is = ClassLoader.getSystemResourceAsStream(sName);
						inp = new BufferedReader(new InputStreamReader(is));
					} catch (Exception e) {
						// e.printStackTrace();
					}
				}
				if (inp != null) {
					String          theLine = inp.readLine(  );
					int i;
					while ( theLine != null ) {
						if ((i = theLine.indexOf("${version}")) != -1) {
							theLine = theLine.substring(0, i)
							+ net.sourceforge.jibs.util.Version.getVersion()
							+ theLine.substring(i+10);
						}
						out.println( theLine );
						theLine = inp.readLine(  );
					}
					inp.close(  );
				} else {
					out.println("jIBS can't read \"" + sName + "\"");
				}
				SimpleDateFormat formatter = new SimpleDateFormat("MMMMMMMMMM dd HH:mm:ss yyyy z",
						Locale.US);
				Date now = new Date();
				String myLoginStr1 = formatter.format(now);

				DateFormat fmt = new SimpleDateFormat("MMMMMMMMMM dd HH:mm:ss yyyy z",
						Locale.US);

				fmt.setTimeZone(TimeZone.getTimeZone("UTC"));

				String myLoginStr2 = fmt.format(now);

				out.println(myLoginStr1 + "        [" + myLoginStr2 + "]");
			}

			out.print("login: ");
			out.flush();
		} catch (FileNotFoundException e) {
			JibsServer.getInstance().logException(e);
		} catch (IOException e) {
			JibsServer.getInstance().logException(e);
		} finally {
			try {
				if (inp != null) {
					inp.close();
				}
			} catch (IOException e) {
				JibsServer.getInstance().logException(e);
			}
		}
	}

	
}
