/******************************************************************************
 * Show_Command - Gimme some data
 * $Id: Show_Command.java,v 1.3 2011/06/24 01:52:06 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Show_Command.java,v $
 * Revision 1.3  2011/06/24 01:52:06  dick
 * Get rid of showWatcher(), it is redundant with showWatchingPlayer().
 *
 * Revision 1.2  2011/06/18 06:23:07  dick
 * Handle the "max" subcommand.
 *
 * Revision 1.1  2011/06/16 19:05:05  dick
 * Implement the "show" command.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.SavedGameParam;

import com.ibatis.sqlmap.client.SqlMapClient;

/** show something about the jIBS system.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @since 2011/06/15
 * @version $Revision: 1.3 $ <br> $Date: 2011/06/24 01:52:06 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Show_Command.java">cvs Show_Command.java</a>
 */
public class Show_Command implements JibsCommand {
	
	@Override
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		Object[] obj;
		String msg;
		if (args.length == 1) {
			// m_show_what=** show what?
			msg = jibsMessages.convert("m_show_what");
			player.println(msg);
			return true;		// db: I don't care for the multitude of exits in a method, but here it works.
		}
		////////////////////////////////////////////////////////
		if (args[1].equals("savedcount")) {
			String name = player.getName();
			if (args.length == 3) {
				name = args[2];
				Player p = jibsServer.getPlayerFromDatabase(name);
				if (p == null) {
					// m_show_saved_count_unknown=** There is no user called '%0'.
					obj = new Object[] { name };
					msg = jibsMessages.convert("m_show_saved_count_unknown", obj);
					player.println(msg);
					return(true);
				}
			}
			Integer count = getSavedCount(jibsServer, name);
			obj = new Object[] {name, count, count==1 ? "" : "s"};
			// m_show_saved_count=%0 has %1 saved game%2.
			msg = jibsMessages.convert("m_show_saved_count", obj);
			player.println(msg);
			return(true);
		}
		////////////////////////////////////////////////////////
		if (args[1].equals("saved")) {
			List<SavedGameParam> list = getSavedMatches(jibsServer, player.getName());
			if (list.isEmpty()) {
				// m_show_saved_none=no saved games.
				msg = jibsMessages.convert("m_show_saved_none");
				player.println(msg);
				return(true);
			}
			// m_show_saved_head=  opponent          matchlength   score (your points first)
			msg = jibsMessages.convert("m_show_saved_head");
			player.println(msg);
			BackgammonBoard board = new BackgammonBoard();
			for (SavedGameParam sgp : list) {
				board.parseBoard(sgp.getBoard());
				StringBuffer sb = new StringBuffer();
				if (sgp.getPlayer_A().equals(player.getName())) {
					appendShowSavedLine(sb, sgp.getPlayer_B(), board.getMatchlength(), 
							board.getPlayerXScore(), board.getPlayerOScore());
				} else {
					appendShowSavedLine(sb, sgp.getPlayer_A(), board.getMatchlength(), 
							board.getPlayerOScore(), board.getPlayerXScore());
				}
				player.println(sb.toString());
			}
			return(true);
		}
		////////////////////////////////////////////////////////
		if (args[1].equals("watchers")) {
			boolean outed = false;
			Player o;
			for (Player p : jibsServer.getAllClients().values()) {
				if ((o = p.getWatchingPlayer()) != null) {
					if (!outed) {
						// m_show_watching_head=Watching players:
						msg = jibsMessages.convert("m_show_watching_head");
						player.println(msg);
						outed = true;
					}
					obj = new Object[] { p.getName(), o.getName()};
					// m_show_watching=%0 is watching %1.
					msg = jibsMessages.convert("m_show_watching", obj);
					player.println(msg);

				}
			}

			return(true);
		}
		////////////////////////////////////////////////////////
		if (args[1].equals("games")) {
			HashMap<String, String> playerMap = new HashMap<String, String>();
			JibsGame game;
			for (Player p : jibsServer.getAllClients().values()) {
				if (playerMap.containsKey(p.getName()))
					continue;
				if ((game = p.getGame()) != null) {
					BackgammonBoard board = game.getBackgammonBoard();
				
					StringBuffer sb = new StringBuffer();
					String nX = board.getPlayerXName();
					String nO = board.getPlayerOName();
					sb.append(nX);
					addSpaces(sb, nX, 16);
					sb.append("-");
					addSpaces(sb, nO, 16);
					sb.append(nO);
					obj = new Object[] { sb.toString(), board.getMatchlength(), 
							board.getPlayerXScore(), board.getPlayerOScore() };
					// m_show_games_line=%0 (%1 point match %2-%3)
					msg = jibsMessages.convert("m_show_games_line", obj);
					player.println(msg);
					playerMap.put(nX, nX);
					playerMap.put(nO, nO);
				}
			}	
			return(true);
		}
		////////////////////////////////////////////////////////
		if (args[1].equals("max")) {
			obj = new Object[] {
					jibsServer.getMaxClientsSeen()
			};
			msg = jibsMessages.convert("m_max_logins", obj);
			player.println(msg);
	
			return(true);
		}
		// m_show_unknown=** Don't know how to show %0
		obj = new Object[] { args[1] };
		msg = jibsMessages.convert("m_show_unknown", obj);
		player.println(msg);
		return true;
	}

	private void appendShowSavedLine(StringBuffer sb, String name, int matchLength, int score0, int score1) {
		JibsServer jibsServer = JibsServer.getInstance();
		Player opp;
		if ((opp = jibsServer.getPlayer(name)) == null) {
			sb.append("  ");
		} else {
			if (opp.getGame() == null && opp.checkToggle("ready"))
				sb.append("**");
			else
				sb.append(" *");
		}
		sb.append(name);
		addSpaces(sb, name, 25-Integer.toString(matchLength).length());
		sb.append(matchLength);
		addSpaces(sb, Integer.toString(matchLength), 17);
		sb.append(score0);
		sb.append(" - ");
		if (score1 < 10)
			sb.append(" ");
		sb.append(score1);

	}
	private Integer getSavedCount(JibsServer jibsServer, String name) {
		SqlMapClient sqlMapClient = jibsServer.getSqlMapInstance();
		SavedGameParam savedGame = new SavedGameParam(name, name);
		Integer count = 0;

		try {
			count = (Integer) sqlMapClient.queryForObject("SavedGames.countSavedGames",
					savedGame);
		} catch (SQLException e) {
			jibsServer.logException(e);
		}
		return(count);

	}
	
	@SuppressWarnings("unchecked")
	private List<SavedGameParam> getSavedMatches(JibsServer jibsServer, String name) {
		SqlMapClient sqlMapClient = jibsServer.getSqlMapInstance();
		SavedGameParam savedGame = new SavedGameParam(name, name);
		List<SavedGameParam> list = null;

		try {
			list = sqlMapClient.queryForList("SavedGames.loadPlayerSavedGames",
					savedGame);
		} catch (SQLException e) {
			jibsServer.logException(e);
		}
		return(list);
	}
	
	private void addSpaces(StringBuffer sb, String s, int count) {
		for (int i = s.length(); i<count; i++)
			sb.append(" ");
	}
}
