/******************************************************************************
 * JibsStackTrace.java - Simple utilities to return the stack trace of an exception as a String.
 * $Id: JibsStackTrace.java,v 1.3 2011/06/11 19:16:20 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsStackTrace.java,v $
 * Revision 1.3  2011/06/11 19:16:20  dick
 * Javadoc.
 *
 * Revision 1.2  2011/05/31 15:25:49  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.util;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;

/** Simple utilities to return the stack trace of an exception as a String.
 * 
 * @author Axel Leucht
 * @author Dick Balaska
 * @since 2005/06/04
 * @version $Revision: 1.3 $ <br> $Date: 2011/06/11 19:16:20 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/util/JibsStackTrace.java">cvs JibsStackTrace.java</a>
 */
public final class JibsStackTrace {

	/** Get the standard Java Stack Trace as a String.
	 * @param aThrowable The Exception to decode
	 * @return A String of the exception with its stack trace.
	 */
	public static String getStackTrace(Throwable aThrowable) {
		final Writer result = new StringWriter();
		final PrintWriter printWriter = new PrintWriter(result);

		aThrowable.printStackTrace(printWriter);

		return result.toString();
	}

	/** Get a String of the StackTrace for the exception.
	 * Customize it a bit...
	 * @param aThrowable The Exception to decode
	 * @return A String of the exception with its stack trace.
	 */
	public static String getCustomStackTrace(Throwable aThrowable) {
		// add the class name and any message passed to constructor
		final StringBuffer result = new StringBuffer("");

		result.append(aThrowable.toString());

		final String NEW_LINE = System.getProperty("line.separator");

		result.append(NEW_LINE);

		// add each element of the stack trace
		StackTraceElement[] elements = aThrowable.getStackTrace();

		for (int i = 0; i < elements.length; i++) {
			StackTraceElement element = elements[i];

			result.append("        at ");
			result.append(element);

			if (i < (elements.length - 1)) {
				result.append(NEW_LINE);
			}
		}

		return result.toString();
	}
}
