/******************************************************************************
 * SavedGameParam - Pojo of saved game data.
 * $Id: SavedGameParam.java,v 1.5 2011/06/16 11:59:09 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: SavedGameParam.java,v $
 * Revision 1.5  2011/06/16 11:59:09  dick
 * Don't try to unwind attributes from a null object.
 *
 * Revision 1.4  2011/06/16 00:46:58  dick
 * The state of crawfordness needs to persist with the saved game.
 *
 * Revision 1.3  2011/06/15 17:24:58  dick
 * Bug 22 - Resume after timeout sometimes fails.
 *
 * Revision 1.2  2011/05/31 15:25:49  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.util;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.backgammon.JibsMatch;
import net.sourceforge.jibs.server.Player;

import java.util.Date;

/** Pojo of saved game data. 
 * 
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.5 $ <br> $Date: 2011/06/16 11:59:09 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/util/SavedGameParam.java">cvs SavedGameParam.java</a>
 */
public class SavedGameParam {
	private Long id;
	private String player_A;
	private String player_B;
	private Date savedDate;
	private String board;
	private Integer matchlength;
	private Integer turn;
	private Integer matchVersion;
	private	Integer	crawfordState;
	private Player player1;
	private Player player2;

	/** This match has not hit a crawford game yet */
	public final static int	crawfordNone = 0;
	/** This is the crawford game in a match */
	public final static int	crawford = 1;
	/** This match has gone post-crawford */
	public final static int	postCrawford = 2;

	/** Default bean constructor
	 */
	public SavedGameParam() {
	}

	/** Construct a SavedGameParam with these two players. <br>
	 * Used for database searches
	 * @param nameX A player in the match
	 * @param nameO Opponent in the match
	 */
	public SavedGameParam(String nameX, String nameO) {
		this.player_A = nameX;
		this.player_B = nameO;
	}

	/** Construct a SavedGameParam with lots of parameters. <br>
	 * Used when one player leaves the match.
	 * @param game
	 * @param playerX
	 * @param playerO
	 * @param backgammonBoard
	 * @param matchlength2
	 * @param turn2
	 * @param matchVersion2
	 * @param date
	 */
	public SavedGameParam(JibsGame game, Player playerX, Player playerO,
			BackgammonBoard backgammonBoard, int matchlength2,
			int turn2, JibsMatch matchVersion2, int crawfordState, Date date) {
		this.player1 = playerX;
		this.player2 = playerO;
		this.player_A = playerX.getName();
		this.player_B = playerO.getName();
		this.savedDate = date;
		this.board = backgammonBoard.getOutputBoard(game);
		this.matchlength = matchlength2;
		this.turn = turn2;
		this.matchVersion = matchVersion2.getVersion();
		this.crawfordState = crawfordState;
	}

	public Date getSavedDate() {
		return savedDate;
	}

	public void setSavedDate(Date savedDate) {
		this.savedDate = savedDate;
	}

	public Integer getTurn() {
		return turn;
	}

	public void setTurn(Integer turn) {
		this.turn = turn;
	}

	public Integer getMatchlength() {
		return matchlength;
	}

	public void setMatchlength(Integer matchlength) {
		this.matchlength = matchlength;
	}

	public Integer getMatchVersion() {
		return matchVersion;
	}

	public void setMatchVersion(Integer matchVersion) {
		this.matchVersion = matchVersion;
	}

	/**
	 * @return the crawfordState
	 */
	public Integer getCrawfordState() {
		return crawfordState;
	}

	/**
	 * @param crawfordState the crawfordState to set
	 */
	public void setCrawfordState(Integer crawfordState) {
		this.crawfordState = crawfordState;
	}

	public String getBoard() {
		return board;
	}

	public void setBoard(String board) {
		this.board = board;
	}

	public String getPlayer_A() {
		return player_A;
	}

	public void setPlayer_A(String playerA) {
		this.player_A = playerA;
	}

	public String getPlayer_B() {
		return player_B;
	}

	public void setPlayer_B(String playerB) {
		this.player_B = playerB;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Player getPlayer1() {
		return player1;
	}

	public Player getPlayer2() {
		return player2;
	}

	public void setPlayer1(Player player) {
		this.player1 = player;
	}

	public void setPlayer2(Player player) {
		this.player2 = player;
	}

	/** Return the player that is playing as X. <br>
	 * XXX: This seems interestingly overly complicated.  Why not A is X, B is O and be done with it?  That is the way they came in.  I'll bet a coverage report would never hit the second test.
	 * @return The Player that is playing as X
	 */
	public Player getPlayerX() {
		if (player1 == null)
			return(null);
		if (player_A.equalsIgnoreCase(player1.getName())) {
			return player1;
		}

		if (player_A.equalsIgnoreCase(player2.getName())) {
			return player2;
		}

		return null;
	}

	public Player getPlayerO() {
		if (player1 == null)
			return(null);
		if (player_B.equalsIgnoreCase(player2.getName())) {
			return player2;
		}

		if (player_B.equalsIgnoreCase(player1.getName())) {
			return player1;
		}

		return null;
	}
}
